unit Score;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls,
  Dialogs, ExtCtrls, ImgList, ComCtrls;

type
    TScore = packed record
        Score: integer;
        Name : ShortString;
        Time : ShortString;
        _OldPos: integer;
    end;
    PScore = ^TScore;

type
  TScoreForm = class(TForm)
    Panel1: TPanel;
    ImgComment: TImage;
    ImglStatus: TImageList;
    LstvRank: TListView;
    Image1: TImage;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Label1Click(Sender: TObject);
  private
    FScores: TList;
    procedure ResetPosition();
    procedure ClearScoreFromLast(const LeftCount: integer);
    procedure DispScoreData();
  public
    procedure DispScore(const NewScore: integer); overload;
    procedure DispScore; overload;
    function GetHiScore(): integer;
  end;

var
    ScoreForm: TScoreForm;

implementation

{$R *.dfm}
{$R Extra.res}

uses
    Info;

const
    MAX_COUNT = 10;

    IMG_NEW  = 0;
    IMG_KEEP = 1;
    IMG_DOWN = 2;
var
    DataFileName: string = '';

//  ---------------------------------------------------------------------------
//  XRǍ݂̏ʂݒ
procedure TScoreForm.ResetPosition();
var
    i: integer;
    Score: PScore;
begin
    for i := 0 to FScores.Count - 1 do
    begin
        Score := PScore(FScores.Items[i]);
        Score^._OldPos := i;
    end;
end;

//  ---------------------------------------------------------------------------
//  XRAf[^w茏cČ납NA
procedure TScoreForm.ClearScoreFromLast(const LeftCount: integer);
var
    Score: PScore;
begin
    while FScores.Count > LeftCount do
    begin
        Score := PScore(FScores.Last);
        Dispose(Score);
        FScores.Delete(FScores.Count - 1);
    end;
end;

//  ---------------------------------------------------------------------------
//  \[gp
function CompareScore(Item1, Item2: Pointer): Integer;
var
    S1, S2: PScore;
begin
    S1 := PScore(Item1);
    S2 := PScore(Item2);
    Result := (S2^.Score - S1^.Score);
    if (Result = 0) then Result := CompareStr(S1^.Time, S2^.Time);
end;

//  ---------------------------------------------------------------------------
//  tH[NGCg
procedure TScoreForm.FormCreate(Sender: TObject);
var
    F: TextFile;
    S, Temp: string;
    SL: TStringList;
    Score: PScore;
begin
    //  ^Cgݒ
    Self.Caption := Application.Title + ' - ' + Self.Name;
    //  IuWFNg쐬
    FScores := TList.Create;
    SL := TStringList.Create;
    try
        //  f[^t@C̃pX
        DataFileName := ChangeFileExt(ParamStr(0), '.dat');
        //  f[^t@CΓǂݍ
        if FileExists(DataFileName) then
        begin
            AssignFile(F, DataFileName);
            Reset(F);
            try
                while not Eof(F) do
                begin
                    Readln(F, S);
                    SL.CommaText := S;
                    if SL.Count > 2 then
                    begin
                        //  \̂mۂ
                        New(Score);
                        //  XRAZbg
                        Temp := SL.Strings[0];
                        Score^.Score := StrToIntDef(Temp, 0);
                        //  ̃Zbg
                        Temp := SL.Strings[1];
                        Score^.Name := Temp;
                        //  Zbg
                        Temp := SL.Strings[2];
                        Score^.Time := Temp;
                        //  Xgɒǉ
                        FScores.Add(Score);
                    end;
                end;
            finally
                CloseFile(F);
            end;
        end;
        //  XRAɃ\[g
        FScores.Sort(CompareScore);
        //  ʃZbg
        ResetPosition();
    finally
        SL.Free;
    end;
end;

//  ---------------------------------------------------------------------------
//  tH[fXgC
procedure TScoreForm.FormDestroy(Sender: TObject);
var
    i: integer;
    F: TextFile;
    Score: PScore;
begin
    //  f[^̏
    AssignFile(F, DataFileName);
    Rewrite(F);
    try
        for i := 0 to FScores.Count - 1 do
        begin
            Score := PScore(FScores.Items[i]);
            Writeln(F, Format('%d,%s,%s', [Score^.Score, Score^.Name, Score^.Time]));
        end;
    finally
        CloseFile(F);
    end;

    //  XRAf[^j
    ClearScoreFromLast(0);
    FScores.Free;
end;

//  ---------------------------------------------------------------------------
//  XRA\
procedure TScoreForm.DispScore(const NewScore: integer);
var
    NewScoreTime: string;
    Score: PScore;
    i, Found: integer;
begin
    //  ݎێ
    NewScoreTime := FormatDateTime('yyyy/mm/dd/hh:nn:ss', Now);
    //  Vzm
    New(Score);
    //  lZbg
    Score^.Score := NewScore;
    Score^.Name := 'UNNAMED';
    Score^.Time := NewScoreTime;
    Score^._OldPos := -1;
    //  Xgɒǉ
    FScores.Add(Score);
    //  \[g
    FScores.Sort(CompareScore);
    //  11ʈȍ~͓z폜
    ClearScoreFromLast(MAX_COUNT);
    //  ̓zcĂ邩H
    Found := MAX_COUNT + 100;
    for i := 0 to FScores.Count - 1 do
    begin
        Score := PScore(FScores.Items[i]);
        if (Score^.Time = NewScoreTime) then
        begin
            Found := i;
            break;
        end;
    end;
    //  O߂Ă炨
    if (Found < MAX_COUNT) then
    begin
        Score := PScore(FScores.Items[Found]);
        Score^.Name := InputBox('RANK IN !', 'INPUT YOUR NAME HERE', Score^.Name);
    end;
    //  ʂɂăRgύX
    if (Found = 0) then
        ImgComment.Picture.Bitmap.LoadFromResourceName(HInstance, 'EXCEL')
    else if (Found < 3) then
        ImgComment.Picture.Bitmap.LoadFromResourceName(HInstance, 'COOL')
    else if (Found < MAX_COUNT) then
        ImgComment.Picture.Bitmap.LoadFromResourceName(HInstance, 'GJOB')
    else
        ImgComment.Picture.Bitmap.LoadFromResourceName(HInstance, 'AGAIN')
    ;
    //  Xgr[NA
    DispScoreData();
    //  Ɖʕ\
    Self.ShowModal();
end;

//  ---------------------------------------------------------------------------
//  XRA\
procedure TScoreForm.DispScore;
begin
    //  Xgr[NA
    DispScoreData();
    //  ʕ\
    Self.ShowModal();
end;

//  ---------------------------------------------------------------------------
//  ʂ鎞
procedure TScoreForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
    //  ʃZbg
    ResetPosition();
end;

//  ---------------------------------------------------------------------------
//  nCXRA擾
function TScoreForm.GetHiScore(): integer;
var
    Score: PScore;
begin
    //  nCXRAXV
    if (FScores.Count > 0) then
    begin
        Score := PScore(FScores.Items[0]);
        Result := Score^.Score;
    end
    else
        Result := 0
    ;
end;

//  ---------------------------------------------------------------------------
//  XRA`
procedure TScoreForm.DispScoreData();
var
    i: integer;
    Score: PScore;
    ListItem: TListItem;
begin
    //  ACeNA
    LstvRank.Items.Clear;
    //  qgc\
    for i := 0 to FScores.Count - 1 do
    begin
        //  f[^o
        Score := PScore(FScores.Items[i]);
        //  ACeǉ
        ListItem := LstvRank.Items.Add();
        ListItem.Caption := Format(' %2d', [i + 1]);
        //  ԃ`FbN
        if (Score^._OldPos = -1) then ListItem.ImageIndex := IMG_NEW
        else if (Score^._OldPos = i) then ListItem.ImageIndex := IMG_KEEP
        else if (Score^._OldPos < i) then ListItem.ImageIndex := IMG_DOWN
        ;
        ListItem.OverlayIndex := ListItem.ImageIndex;
        ListItem.StateIndex := ListItem.ImageIndex;
        //  ̑f[^
        ListItem.SubItems.Add(Format('%6.6d', [Score^.Score]));
        ListItem.SubItems.Add(Score^.Name);
        ListItem.SubItems.Add(Score^.Time);
    end;
end;

procedure TScoreForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
    if (Key = VK_ESCAPE) then Close();
end;

procedure TScoreForm.Label1Click(Sender: TObject);
begin
    OpenInfoForm();
end;

end.

